@@ -35,18 +35,6 @@
 #include "config.h"
 #include "private.h"
 
-/* to read CAIRO_PS_LEVEL_* constants */
-#ifdef CAIRO_HAS_PS_SURFACE
-#  include <cairo-ps.h>
-#endif
-/* to read CAIRO_PDF_VERSION_* constants */
-#ifdef CAIRO_HAS_PDF_SURFACE
-#  include <cairo-pdf.h>
-#endif
-/* to read CAIRO_SVG_VERSION_* constants */
-#ifdef CAIRO_HAS_SVG_SURFACE
-#  include <cairo-svg.h>
-#endif
 
 /* for XCB api */
 #if defined(CAIRO_HAS_XCB_SURFACE) && defined(HAVE_XPYB)
@@ -272,10 +260,22 @@ PYCAIRO_MOD_INIT(_cairo)
     return PYCAIRO_MOD_ERROR_VAL;
   if (PyType_Ready(&PycairoRadialGradient_Type) < 0)
     return PYCAIRO_MOD_ERROR_VAL;
+  if (PyType_Ready(&PycairoMeshPattern_Type) < 0)
+    return PYCAIRO_MOD_ERROR_VAL;
 
   if (PyType_Ready(&PycairoRectangleInt_Type) < 0)
     return PYCAIRO_MOD_ERROR_VAL;
 
+  if (PyType_Ready(&PycairoDevice_Type) < 0)
+    return PYCAIRO_MOD_ERROR_VAL;
+
+#ifdef CAIRO_HAS_SCRIPT_SURFACE
+  if (PyType_Ready(&PycairoScriptDevice_Type) < 0)
+    return PYCAIRO_MOD_ERROR_VAL;
+  if (PyType_Ready(&PycairoScriptSurface_Type) < 0)
+    return PYCAIRO_MOD_ERROR_VAL;
+#endif
+
   if (PyType_Ready(&PycairoRegion_Type) < 0)
     return PYCAIRO_MOD_ERROR_VAL;
 
@@ -318,6 +318,10 @@ PYCAIRO_MOD_INIT(_cairo)
   if (PyType_Ready(&PycairoXlibSurface_Type) < 0)
     return PYCAIRO_MOD_ERROR_VAL;
 #endif
+#ifdef CAIRO_HAS_TEE_SURFACE
+  if (PyType_Ready(&PycairoTeeSurface_Type) < 0)
+    return PYCAIRO_MOD_ERROR_VAL;
+#endif
 
 #if PY_MAJOR_VERSION < 3
   m = Py_InitModule("cairo._cairo", cairo_functions);
@@ -327,6 +331,12 @@ PYCAIRO_MOD_INIT(_cairo)
   if (m == NULL)
     return PYCAIRO_MOD_ERROR_VAL;
 
+  if(init_buffer_proxy() < 0)
+    return PYCAIRO_MOD_ERROR_VAL;
+
+  if(init_enums(m) < 0)
+    return PYCAIRO_MOD_ERROR_VAL;
+
   PyModule_AddStringConstant(m, "version", VERSION);
   PyModule_AddObject(m, "version_info",
 		     Py_BuildValue("(iii)",
@@ -365,6 +375,9 @@ PYCAIRO_MOD_INIT(_cairo)
   Py_INCREF(&PycairoRadialGradient_Type);
   PyModule_AddObject(m, "RadialGradient",
 		     (PyObject *)&PycairoRadialGradient_Type);
+  Py_INCREF(&PycairoRadialGradient_Type);
+  PyModule_AddObject(m, "MeshPattern",
+                     (PyObject *)&PycairoMeshPattern_Type);
 
   Py_INCREF(&PycairoRectangleInt_Type);
   PyModule_AddObject(m, "RectangleInt",  (PyObject *)&PycairoRectangleInt_Type);
@@ -378,6 +391,16 @@ PYCAIRO_MOD_INIT(_cairo)
   Py_INCREF(&PycairoSurface_Type);
   PyModule_AddObject(m, "Surface", (PyObject *)&PycairoSurface_Type);
 
+  Py_INCREF(&PycairoDevice_Type);
+  PyModule_AddObject(m, "Device", (PyObject *)&PycairoDevice_Type);
+
+#ifdef CAIRO_HAS_SCRIPT_SURFACE
+  Py_INCREF(&PycairoScriptDevice_Type);
+  PyModule_AddObject(m, "ScriptDevice", (PyObject *)&PycairoScriptDevice_Type);
+  Py_INCREF(&PycairoScriptSurface_Type);
+  PyModule_AddObject(m, "ScriptSurface", (PyObject *)&PycairoScriptSurface_Type);
+#endif
+
 #ifdef CAIRO_HAS_IMAGE_SURFACE
   Py_INCREF(&PycairoImageSurface_Type);
   PyModule_AddObject(m, "ImageSurface",
@@ -426,6 +449,12 @@ PYCAIRO_MOD_INIT(_cairo)
 		     (PyObject *)&PycairoXlibSurface_Type);
 #endif
 
+#ifdef CAIRO_HAS_TEE_SURFACE
+  Py_INCREF(&PycairoTeeSurface_Type);
+  PyModule_AddObject(m, "TeeSurface",
+		     (PyObject *)&PycairoTeeSurface_Type);
+#endif
+
   /* Add 'cairo.Error' to the module */
 #if PY_MAJOR_VERSION >= 3
   GETSTATE(m)->ErrorObject = error_get_type();
@@ -533,172 +562,12 @@ PYCAIRO_MOD_INIT(_cairo)
 #else
   PyModule_AddIntConstant(m, "HAS_MIME_SURFACE", 0);
 #endif
-
-#define CONSTANT(x) PyModule_AddIntConstant(m, #x, CAIRO_##x)
-  CONSTANT(ANTIALIAS_DEFAULT);
-  CONSTANT(ANTIALIAS_NONE);
-  CONSTANT(ANTIALIAS_GRAY);
-  CONSTANT(ANTIALIAS_SUBPIXEL);
-  CONSTANT(ANTIALIAS_FAST);
-  CONSTANT(ANTIALIAS_GOOD);
-  CONSTANT(ANTIALIAS_BEST);
-
-  CONSTANT(CONTENT_COLOR);
-  CONSTANT(CONTENT_ALPHA);
-  CONSTANT(CONTENT_COLOR_ALPHA);
-
-  CONSTANT(EXTEND_NONE);
-  CONSTANT(EXTEND_REPEAT);
-  CONSTANT(EXTEND_REFLECT);
-  CONSTANT(EXTEND_PAD);
-
-  CONSTANT(FILL_RULE_WINDING);
-  CONSTANT(FILL_RULE_EVEN_ODD);
-
-  CONSTANT(FILTER_FAST);
-  CONSTANT(FILTER_GOOD);
-  CONSTANT(FILTER_BEST);
-  CONSTANT(FILTER_NEAREST);
-  CONSTANT(FILTER_BILINEAR);
-  CONSTANT(FILTER_GAUSSIAN);
-
-  CONSTANT(FONT_WEIGHT_NORMAL);
-  CONSTANT(FONT_WEIGHT_BOLD);
-
-  CONSTANT(FONT_SLANT_NORMAL);
-  CONSTANT(FONT_SLANT_ITALIC);
-  CONSTANT(FONT_SLANT_OBLIQUE);
-
-  CONSTANT(FORMAT_INVALID);
-  CONSTANT(FORMAT_ARGB32);
-  CONSTANT(FORMAT_RGB24);
-  CONSTANT(FORMAT_A8);
-  CONSTANT(FORMAT_A1);
-  CONSTANT(FORMAT_RGB16_565);
-  CONSTANT(FORMAT_RGB30);
-
-  CONSTANT(HINT_METRICS_DEFAULT);
-  CONSTANT(HINT_METRICS_OFF);
-  CONSTANT(HINT_METRICS_ON);
-
-  CONSTANT(HINT_STYLE_DEFAULT);
-  CONSTANT(HINT_STYLE_NONE);
-  CONSTANT(HINT_STYLE_SLIGHT);
-  CONSTANT(HINT_STYLE_MEDIUM);
-  CONSTANT(HINT_STYLE_FULL);
-
-  CONSTANT(LINE_CAP_BUTT);
-  CONSTANT(LINE_CAP_ROUND);
-  CONSTANT(LINE_CAP_SQUARE);
-
-  CONSTANT(LINE_JOIN_MITER);
-  CONSTANT(LINE_JOIN_ROUND);
-  CONSTANT(LINE_JOIN_BEVEL);
-
-  CONSTANT(OPERATOR_CLEAR);
-
-  CONSTANT(OPERATOR_SOURCE);
-  CONSTANT(OPERATOR_OVER);
-  CONSTANT(OPERATOR_IN);
-  CONSTANT(OPERATOR_OUT);
-  CONSTANT(OPERATOR_ATOP);
-
-  CONSTANT(OPERATOR_DEST);
-  CONSTANT(OPERATOR_DEST_OVER);
-  CONSTANT(OPERATOR_DEST_IN);
-  CONSTANT(OPERATOR_DEST_OUT);
-  CONSTANT(OPERATOR_DEST_ATOP);
-
-  CONSTANT(OPERATOR_XOR);
-  CONSTANT(OPERATOR_ADD);
-  CONSTANT(OPERATOR_SATURATE);
-
-  CONSTANT(OPERATOR_MULTIPLY);
-  CONSTANT(OPERATOR_SCREEN);
-  CONSTANT(OPERATOR_OVERLAY);
-  CONSTANT(OPERATOR_DARKEN);
-  CONSTANT(OPERATOR_LIGHTEN);
-  CONSTANT(OPERATOR_COLOR_DODGE);
-  CONSTANT(OPERATOR_COLOR_BURN);
-  CONSTANT(OPERATOR_HARD_LIGHT);
-  CONSTANT(OPERATOR_SOFT_LIGHT);
-  CONSTANT(OPERATOR_DIFFERENCE);
-  CONSTANT(OPERATOR_EXCLUSION);
-  CONSTANT(OPERATOR_HSL_HUE);
-  CONSTANT(OPERATOR_HSL_SATURATION);
-  CONSTANT(OPERATOR_HSL_COLOR);
-  CONSTANT(OPERATOR_HSL_LUMINOSITY);
-
-  CONSTANT(PATH_MOVE_TO);
-  CONSTANT(PATH_LINE_TO);
-  CONSTANT(PATH_CURVE_TO);
-  CONSTANT(PATH_CLOSE_PATH);
-
-#ifdef CAIRO_HAS_PDF_SURFACE
-  CONSTANT(PDF_VERSION_1_4);
-  CONSTANT(PDF_VERSION_1_5);
+#ifdef CAIRO_HAS_SCRIPT_SURFACE
+  PyModule_AddIntConstant(m, "HAS_SCRIPT_SURFACE", 1);
+#else
+  PyModule_AddIntConstant(m, "HAS_SCRIPT_SURFACE", 0);
 #endif
 
-  CONSTANT(REGION_OVERLAP_IN);
-  CONSTANT(REGION_OVERLAP_OUT);
-  CONSTANT(REGION_OVERLAP_PART);
-
-#ifdef CAIRO_HAS_PS_SURFACE
-  CONSTANT(PS_LEVEL_2);
-  CONSTANT(PS_LEVEL_3);
-#endif
-
-#ifdef CAIRO_HAS_SVG_SURFACE
-  CONSTANT(SVG_VERSION_1_1);
-  CONSTANT(SVG_VERSION_1_2);
-#endif
-
-  CONSTANT(SUBPIXEL_ORDER_DEFAULT);
-  CONSTANT(SUBPIXEL_ORDER_RGB);
-  CONSTANT(SUBPIXEL_ORDER_BGR);
-  CONSTANT(SUBPIXEL_ORDER_VRGB);
-  CONSTANT(SUBPIXEL_ORDER_VBGR);
-
-  CONSTANT(STATUS_SUCCESS);
-  CONSTANT(STATUS_NO_MEMORY);
-  CONSTANT(STATUS_INVALID_RESTORE);
-  CONSTANT(STATUS_INVALID_POP_GROUP);
-  CONSTANT(STATUS_NO_CURRENT_POINT);
-  CONSTANT(STATUS_INVALID_MATRIX);
-  CONSTANT(STATUS_INVALID_STATUS);
-  CONSTANT(STATUS_NULL_POINTER);
-  CONSTANT(STATUS_INVALID_STRING);
-  CONSTANT(STATUS_INVALID_PATH_DATA);
-  CONSTANT(STATUS_READ_ERROR);
-  CONSTANT(STATUS_WRITE_ERROR);
-  CONSTANT(STATUS_SURFACE_FINISHED);
-  CONSTANT(STATUS_SURFACE_TYPE_MISMATCH);
-  CONSTANT(STATUS_PATTERN_TYPE_MISMATCH);
-  CONSTANT(STATUS_INVALID_CONTENT);
-  CONSTANT(STATUS_INVALID_FORMAT);
-  CONSTANT(STATUS_INVALID_VISUAL);
-  CONSTANT(STATUS_FILE_NOT_FOUND);
-  CONSTANT(STATUS_INVALID_DASH);
-  CONSTANT(STATUS_INVALID_DSC_COMMENT);
-  CONSTANT(STATUS_INVALID_INDEX);
-  CONSTANT(STATUS_CLIP_NOT_REPRESENTABLE);
-  CONSTANT(STATUS_TEMP_FILE_ERROR);
-  CONSTANT(STATUS_INVALID_STRIDE);
-  CONSTANT(STATUS_FONT_TYPE_MISMATCH);
-  CONSTANT(STATUS_USER_FONT_IMMUTABLE);
-  CONSTANT(STATUS_USER_FONT_ERROR);
-  CONSTANT(STATUS_NEGATIVE_COUNT);
-  CONSTANT(STATUS_INVALID_CLUSTERS);
-  CONSTANT(STATUS_INVALID_SLANT);
-  CONSTANT(STATUS_INVALID_WEIGHT);
-  CONSTANT(STATUS_INVALID_SIZE);
-  CONSTANT(STATUS_USER_FONT_NOT_IMPLEMENTED);
-  CONSTANT(STATUS_DEVICE_TYPE_MISMATCH);
-  CONSTANT(STATUS_DEVICE_ERROR);
-  CONSTANT(STATUS_INVALID_MESH_CONSTRUCTION);
-  CONSTANT(STATUS_DEVICE_FINISHED);
-  CONSTANT(STATUS_LAST_STATUS);
-
 #define STRCONSTANT(x) PyModule_AddStringConstant(m, #x, CAIRO_##x)
 
   STRCONSTANT(MIME_TYPE_JP2);
@@ -708,7 +577,6 @@ PYCAIRO_MOD_INIT(_cairo)
   STRCONSTANT(MIME_TYPE_UNIQUE_ID);
 
 #undef STRCONSTANT
-#undef CONSTANT
 
 #if PY_MAJOR_VERSION >= 3
   /* Create a Capsule containing the CAPI pointer */
