@@ -112,6 +112,16 @@ PycairoSurface_FromSurface (cairo_surface_t *surface, PyObject *base) {
   case CAIRO_SURFACE_TYPE_XLIB:
     type = &PycairoXlibSurface_Type;
     break;
+#endif
+#if CAIRO_HAS_SCRIPT_SURFACE
+  case CAIRO_SURFACE_TYPE_SCRIPT:
+    type = &PycairoScriptSurface_Type;
+    break;
+#endif
+#if CAIRO_HAS_TEE_SURFACE
+  case CAIRO_SURFACE_TYPE_TEE:
+    type = &PycairoTeeSurface_Type;
+    break;
 #endif
   default:
     type = &PycairoSurface_Type;
@@ -242,7 +252,7 @@ surface_flush (PycairoSurface *o) {
 
 static PyObject *
 surface_get_content (PycairoSurface *o) {
-  return PYCAIRO_PyLong_FromLong (cairo_surface_get_content (o->surface));
+  RETURN_INT_ENUM (Content, cairo_surface_get_content (o->surface));
 }
 
 static PyObject *
@@ -252,6 +262,13 @@ surface_get_device_offset (PycairoSurface *o) {
   return Py_BuildValue("(dd)", x_offset, y_offset);
 }
 
+static PyObject *
+surface_get_device_scale (PycairoSurface *o) {
+  double x_scale, y_scale;
+  cairo_surface_get_device_scale (o->surface, &x_scale, &y_scale);
+  return Py_BuildValue("(dd)", x_scale, y_scale);
+}
+
 static PyObject *
 surface_get_fallback_resolution (PycairoSurface *o) {
   double x_ppi, y_ppi;
@@ -268,6 +285,14 @@ surface_get_font_options (PycairoSurface *o) {
   return PycairoFontOptions_FromFontOptions (options);
 }
 
+static PyObject *
+surface_get_device (PycairoSurface *o) {
+  cairo_device_t *device = cairo_surface_get_device (o->surface);
+  if (device == NULL)
+    Py_RETURN_NONE;
+  return PycairoDevice_FromDevice (cairo_device_reference (device));
+}
+
 static PyObject *
 surface_mark_dirty (PycairoSurface *o) {
   cairo_surface_mark_dirty (o->surface);
@@ -300,6 +325,26 @@ surface_set_device_offset (PycairoSurface *o, PyObject *args) {
   Py_RETURN_NONE;
 }
 
+static PyObject *
+surface_set_device_scale (PycairoSurface *o, PyObject *args) {
+  double x_scale, y_scale;
+  cairo_matrix_t transform;
+
+  if (!PyArg_ParseTuple (args, "dd:Surface.set_device_scale",
+			 &x_scale, &y_scale))
+    return NULL;
+
+  /* cairo asserts the following without reporting an error back.
+   * Since we don't want things to crash in Python replicate the logic here.
+   */
+  cairo_matrix_init_scale (&transform, x_scale, y_scale);
+  RETURN_NULL_IF_CAIRO_ERROR (cairo_matrix_invert (&transform));
+
+  cairo_surface_set_device_scale (o->surface, x_scale, y_scale);
+  Py_RETURN_NONE;
+
+}
+
 static PyObject *
 surface_set_fallback_resolution (PycairoSurface *o, PyObject *args) {
   double x_ppi, y_ppi;
@@ -546,13 +591,16 @@ static PyMethodDef surface_methods[] = {
   {"flush",          (PyCFunction)surface_flush,              METH_NOARGS},
   {"get_content",    (PyCFunction)surface_get_content,        METH_NOARGS},
   {"get_device_offset",(PyCFunction)surface_get_device_offset,METH_NOARGS},
+  {"get_device_scale", (PyCFunction)surface_get_device_scale, METH_NOARGS},
   {"get_fallback_resolution",(PyCFunction)surface_get_fallback_resolution,
    METH_NOARGS},
   {"get_font_options",(PyCFunction)surface_get_font_options,  METH_NOARGS},
+  {"get_device",     (PyCFunction)surface_get_device,         METH_NOARGS},
   {"mark_dirty",     (PyCFunction)surface_mark_dirty,         METH_NOARGS},
   {"mark_dirty_rectangle", (PyCFunction)surface_mark_dirty_rectangle,
    METH_VARARGS},
   {"set_device_offset",(PyCFunction)surface_set_device_offset,METH_VARARGS},
+  {"set_device_scale", (PyCFunction)surface_set_device_scale, METH_VARARGS},
   {"set_fallback_resolution",(PyCFunction)surface_set_fallback_resolution,
    METH_VARARGS},
   {"show_page",      (PyCFunction)surface_show_page,          METH_NOARGS},
@@ -765,7 +813,6 @@ image_surface_format_stride_for_width (PyObject *self, PyObject *args) {
 static PyObject *
 image_surface_get_data (PycairoImageSurface *o) {
 #if PY_MAJOR_VERSION >= 3
-  Py_buffer view;
   cairo_surface_t *surface;
   int height, stride;
   unsigned char * buffer;
@@ -777,10 +824,8 @@ image_surface_get_data (PycairoImageSurface *o) {
   }
   height = cairo_image_surface_get_height (surface);
   stride = cairo_image_surface_get_stride (surface);
-  if (PyBuffer_FillInfo (&view, (PyObject *)o, buffer, height * stride, 0, 0) < 0)
-    return NULL;
 
-  return PyMemoryView_FromBuffer (&view);
+  return buffer_proxy_create_view((PyObject *)o, buffer, height * stride, 0);
 #else
   return PyBuffer_FromReadWriteObject((PyObject *)o, 0, Py_END_OF_BUFFER);
 #endif
@@ -788,7 +833,7 @@ image_surface_get_data (PycairoImageSurface *o) {
 
 static PyObject *
 image_surface_get_format (PycairoImageSurface *o) {
-  return PYCAIRO_PyLong_FromLong (cairo_image_surface_get_format (o->surface));
+  RETURN_INT_ENUM (Format, cairo_image_surface_get_format (o->surface));
 }
 
 static PyObject *
@@ -809,9 +854,9 @@ image_surface_get_width (PycairoImageSurface *o) {
 #if PY_MAJOR_VERSION < 3
 
 /* Buffer interface functions, used by ImageSurface.get_data() */
-static int
-image_surface_buffer_getreadbuf (PycairoImageSurface *o, int segment,
-				 const void **ptr) {
+static Py_ssize_t
+image_surface_buffer_getreadbuf (PycairoImageSurface *o, Py_ssize_t segment,
+                                 const void **ptr) {
   cairo_surface_t *surface = o->surface;
   int height, stride;
 
@@ -826,9 +871,9 @@ image_surface_buffer_getreadbuf (PycairoImageSurface *o, int segment,
   return height * stride;
 }
 
-static int
-image_surface_buffer_getwritebuf (PycairoImageSurface *o, int segment,
-				  const void **ptr) {
+static Py_ssize_t
+image_surface_buffer_getwritebuf (PycairoImageSurface *o, Py_ssize_t segment,
+                                  const void **ptr) {
   cairo_surface_t *surface = o->surface;
   int height, stride;
 
@@ -843,8 +888,8 @@ image_surface_buffer_getwritebuf (PycairoImageSurface *o, int segment,
   return height * stride;
 }
 
-static int
-image_surface_buffer_getsegcount (PycairoImageSurface *o, int *lenp) {
+static Py_ssize_t
+image_surface_buffer_getsegcount (PycairoImageSurface *o, Py_ssize_t *lenp) {
   if (lenp) {
     /* report the sum of the sizes (in bytes) of all segments */
     cairo_surface_t *surface = o->surface;
@@ -855,12 +900,28 @@ image_surface_buffer_getsegcount (PycairoImageSurface *o, int *lenp) {
   return 1;  /* surface data is all in one segment */
 }
 
+static Py_ssize_t
+image_surface_buffer_getcharbuffer (PycairoImageSurface *o,
+                                    Py_ssize_t segment,
+                                    char **ptrptr) {
+  Py_ssize_t segment_size;
+
+  if (segment != 0) {
+    PyErr_SetString(PyExc_SystemError,
+      "accessing non-existent ImageSurface segment");
+    return -1;
+  }
+
+  image_surface_buffer_getsegcount (o, &segment_size);
+  return segment_size;
+}
+
 /* See Python C API Manual 10.7 */
 static PyBufferProcs image_surface_as_buffer = {
   (readbufferproc) image_surface_buffer_getreadbuf,
   (writebufferproc)image_surface_buffer_getwritebuf,
   (segcountproc)   image_surface_buffer_getsegcount,
-  (charbufferproc) NULL,
+  (charbufferproc) image_surface_buffer_getcharbuffer,
 };
 #endif
 
@@ -907,7 +968,12 @@ PyTypeObject PycairoImageSurface_Type = {
 #else
   0,                                  /* tp_as_buffer */
 #endif
+#if PY_MAJOR_VERSION < 3
+  Py_TPFLAGS_DEFAULT |
+     Py_TPFLAGS_HAVE_GETCHARBUFFER,   /* tp_flags */
+#else
   Py_TPFLAGS_DEFAULT,                 /* tp_flags */
+#endif
   0,                                  /* tp_doc */
   0,                                  /* tp_traverse */
   0,                                  /* tp_clear */
@@ -1019,7 +1085,7 @@ pdf_get_versions (PyObject *self) {
     return NULL;
 
   for (i=0; i < num_versions; i++) {
-    num = PYCAIRO_PyLong_FromLong (versions[i]);
+    num = CREATE_INT_ENUM (PDFVersion, versions[i]);
     if (num == NULL) {
       Py_DECREF (list);
       return NULL;
@@ -1120,6 +1186,93 @@ PyTypeObject PycairoPDFSurface_Type = {
 };
 #endif /* CAIRO_HAS_PDF_SURFACE */
 
+#ifdef CAIRO_HAS_SCRIPT_SURFACE
+#include <cairo-script.h>
+
+typedef PycairoSurface PycairoScriptSurface;
+
+static PyObject *
+script_surface_new (PyTypeObject *type, PyObject *args, PyObject *kwds) {
+  cairo_content_t content;
+  double width, height;
+  PyObject *pydevice;
+
+  if (!PyArg_ParseTuple (args, "O!idd:ScriptSurface.__new__",
+      &PycairoScriptDevice_Type, &pydevice, &content, &width, &height))
+    return NULL;
+
+  return PycairoSurface_FromSurface (
+    cairo_script_surface_create (
+      ((PycairoDevice*)pydevice)->device, content, width, height),
+    NULL);
+}
+
+static PyObject *
+script_surface_create_for_target (PyTypeObject *type, PyObject *args) {
+  PyObject *pydevice, *target;
+
+  if (!PyArg_ParseTuple (args, "O!O!:ScriptSurface.create_for_target",
+        &PycairoScriptDevice_Type, &pydevice, &PycairoSurface_Type, &target))
+    return NULL;
+
+  return PycairoSurface_FromSurface (
+    cairo_script_surface_create_for_target (
+      ((PycairoDevice*)pydevice)->device, ((PycairoSurface*)target)->surface),
+    NULL);
+}
+
+static PyMethodDef script_surface_methods[] = {
+  {"create_for_target",
+    (PyCFunction)script_surface_create_for_target, METH_VARARGS | METH_CLASS},
+  {NULL, NULL, 0, NULL},
+};
+
+PyTypeObject PycairoScriptSurface_Type = {
+  PyVarObject_HEAD_INIT(NULL, 0)
+  "cairo.ScriptSurface",              /* tp_name */
+  sizeof(PycairoScriptSurface),      /* tp_basicsize */
+  0,                                  /* tp_itemsize */
+  0,                                  /* tp_dealloc */
+  0,                                  /* tp_print */
+  0,                                  /* tp_getattr */
+  0,                                  /* tp_setattr */
+  0,                                  /* tp_compare */
+  0,                                  /* tp_repr */
+  0,                                  /* tp_as_number */
+  0,                                  /* tp_as_sequence */
+  0,                                  /* tp_as_mapping */
+  0,                                  /* tp_hash */
+  0,                                  /* tp_call */
+  0,                                  /* tp_str */
+  0,                                  /* tp_getattro */
+  0,                                  /* tp_setattro */
+  0,                                  /* tp_as_buffer */
+  Py_TPFLAGS_DEFAULT,                 /* tp_flags */
+  0,                                  /* tp_doc */
+  0,                                  /* tp_traverse */
+  0,                                  /* tp_clear */
+  0,                                  /* tp_richcompare */
+  0,                                  /* tp_weaklistoffset */
+  0,                                  /* tp_iter */
+  0,                                  /* tp_iternext */
+  script_surface_methods,             /* tp_methods */
+  0,                                  /* tp_members */
+  0,                                  /* tp_getset */
+  &PycairoSurface_Type,               /* tp_base */
+  0,                                  /* tp_dict */
+  0,                                  /* tp_descr_get */
+  0,                                  /* tp_descr_set */
+  0,                                  /* tp_dictoffset */
+  0,                                  /* tp_init */
+  0,                                  /* tp_alloc */
+  (newfunc)script_surface_new,        /* tp_new */
+  0,                                  /* tp_free */
+  0,                                  /* tp_is_gc */
+  0,                                  /* tp_bases */
+};
+
+#endif /* CAIRO_HAS_SCRIPT_SURFACE */
+
 
 /* Class PSSurface(Surface) ----------------------------------------------- */
 #ifdef CAIRO_HAS_PS_SURFACE
@@ -1275,7 +1428,7 @@ ps_get_levels (PyObject *self) {
     return NULL;
 
   for (i=0; i < num_levels; i++) {
-    num = PYCAIRO_PyLong_FromLong (levels[i]);
+    num = CREATE_INT_ENUM (PSLevel, levels[i]);
     if (num == NULL) {
       Py_DECREF (list);
       return NULL;
@@ -1532,7 +1685,7 @@ svg_get_versions (PyObject *self) {
     return NULL;
 
   for (i=0; i < num_versions; i++) {
-    num = PYCAIRO_PyLong_FromLong (versions[i]);
+    num = CREATE_INT_ENUM (SVGVersion, versions[i]);
     if (num == NULL) {
       Py_DECREF (list);
       return NULL;
@@ -1993,3 +2146,111 @@ PyTypeObject PycairoXlibSurface_Type = {
   0,                                  /* tp_bases */
 };
 #endif  /* CAIRO_HAS_XLIB_SURFACE */
+
+#ifdef CAIRO_HAS_TEE_SURFACE
+#include <cairo-tee.h>
+
+static PyObject *
+tee_surface_new (PyTypeObject *type, PyObject *args, PyObject *kwds) {
+  PyObject *pysurface;
+
+  if (!PyArg_ParseTuple (args, "O!:TeeSurface.__new__",
+      &PycairoSurface_Type, &pysurface))
+    return NULL;
+
+  return PycairoSurface_FromSurface (
+    cairo_tee_surface_create(((PycairoSurface*)pysurface)->surface),
+    NULL);
+}
+
+static PyObject *
+tee_surface_add (PycairoTeeSurface *obj, PyObject *args) {
+  PyObject *pysurface;
+
+  if (!PyArg_ParseTuple(args, "O!:TeeSurface.add",
+      &PycairoSurface_Type, &pysurface))
+    return NULL;
+
+  cairo_tee_surface_add (obj->surface, ((PycairoSurface*)pysurface)->surface);
+  RETURN_NULL_IF_CAIRO_SURFACE_ERROR (obj->surface);
+
+  Py_RETURN_NONE;
+}
+
+static PyObject *
+tee_surface_remove (PycairoTeeSurface *obj, PyObject *args) {
+  PyObject *pysurface;
+
+  if (!PyArg_ParseTuple(args, "O!:TeeSurface.remove",
+      &PycairoSurface_Type, &pysurface))
+    return NULL;
+
+  cairo_tee_surface_remove (obj->surface, ((PycairoSurface*)pysurface)->surface);
+  RETURN_NULL_IF_CAIRO_SURFACE_ERROR (obj->surface);
+
+  Py_RETURN_NONE;
+}
+
+static PyObject *
+tee_surface_index (PycairoTeeSurface *obj, PyObject *args) {
+  unsigned int index;
+
+  if (!PyArg_ParseTuple(args, "I:TeeSurface.index", &index))
+    return NULL;
+
+  return PycairoSurface_FromSurface (
+    cairo_surface_reference (cairo_tee_surface_index (obj->surface, index)),
+    NULL);
+}
+
+static PyMethodDef tee_surface_methods[] = {
+  {"add",    (PyCFunction)tee_surface_add,    METH_VARARGS },
+  {"remove", (PyCFunction)tee_surface_remove, METH_VARARGS },
+  {"index",  (PyCFunction)tee_surface_index,  METH_VARARGS },
+  {NULL, NULL, 0, NULL},
+};
+
+PyTypeObject PycairoTeeSurface_Type = {
+  PyVarObject_HEAD_INIT(NULL, 0)
+  "cairo.TeeSurface",                 /* tp_name */
+  sizeof(PycairoTeeSurface),          /* tp_basicsize */
+  0,                                  /* tp_itemsize */
+  0,                                  /* tp_dealloc */
+  0,                                  /* tp_print */
+  0,                                  /* tp_getattr */
+  0,                                  /* tp_setattr */
+  0,                                  /* tp_compare */
+  0,                                  /* tp_repr */
+  0,                                  /* tp_as_number */
+  0,                                  /* tp_as_sequence */
+  0,                                  /* tp_as_mapping */
+  0,                                  /* tp_hash */
+  0,                                  /* tp_call */
+  0,                                  /* tp_str */
+  0,                                  /* tp_getattro */
+  0,                                  /* tp_setattro */
+  0,                                  /* tp_as_buffer */
+  Py_TPFLAGS_DEFAULT,                 /* tp_flags */
+  0,                                  /* tp_doc */
+  0,                                  /* tp_traverse */
+  0,                                  /* tp_clear */
+  0,                                  /* tp_richcompare */
+  0,                                  /* tp_weaklistoffset */
+  0,                                  /* tp_iter */
+  0,                                  /* tp_iternext */
+  tee_surface_methods,                /* tp_methods */
+  0,                                  /* tp_members */
+  0,                                  /* tp_getset */
+  &PycairoSurface_Type,               /* tp_base */
+  0,                                  /* tp_dict */
+  0,                                  /* tp_descr_get */
+  0,                                  /* tp_descr_set */
+  0,                                  /* tp_dictoffset */
+  0,                                  /* tp_init */
+  0,                                  /* tp_alloc */
+  (newfunc)tee_surface_new,           /* tp_new */
+  0,                                  /* tp_free */
+  0,                                  /* tp_is_gc */
+  0,                                  /* tp_bases */
+};
+#endif  /* CAIRO_HAS_TEE_SURFACE */
