@@ -11,7 +11,7 @@ intersphinx_mapping = {
 source_suffix = '.rst'
 master_doc = 'index'
 project = u'Pycairo'
-copyright = u'2008, Steve Chaplin'
+html_show_copyright = False
 exclude_patterns = ['_build']
 pygments_style = 'sphinx'
 html_theme = "sphinx_rtd_theme"
@@ -32,4 +32,6 @@ extlinks = {
     'fdobug': ('https://bugs.freedesktop.org/show_bug.cgi?id=%s', '#fdo-'),
     'bug': ('https://github.com/pygobject/pycairo/issues/%s', '#'),
     'pr': ('https://github.com/pygobject/pycairo/pull/%s', '#pr-'),
+    'user': ('https://github.com/%s', ''),
 }
+suppress_warnings = ["image.nonlocal_uri"]
