@@ -4,11 +4,13 @@ from __future__ import division
 from __future__ import absolute_import
 from __future__ import print_function
 
+import os
 import io
 import sys
 import tempfile as tfi
 import base64
 import zlib
+import shutil
 
 import cairo
 import pytest
@@ -20,6 +22,38 @@ except NameError:
     long = int
 
 
+def test_show_unicode_text():
+    width, height = 300, 300
+    surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
+    ctx = cairo.Context(surface)
+
+    ctx.scale(width, height)
+    ctx.set_line_width(0.04)
+
+    ctx.select_font_face(
+        "Sans", cairo.FONT_SLANT_NORMAL, cairo.FONT_WEIGHT_NORMAL)
+    ctx.set_font_size(0.20)
+    ctx.move_to(0.05, 0.5)
+    ctx.show_text(u"ēxāmple.")
+
+
+def test_unicode_filenames():
+    # FIXME: cairo does not support wchar on Windows and byte support is
+    # missing under Python 3
+
+    surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 10, 10)
+    dirname = tfi.mkdtemp()
+    old_dir = os.getcwd()
+    try:
+        os.chdir(dirname)
+        surface.write_to_png("foobar")
+        new = cairo.ImageSurface.create_from_png(u"foobar")
+        assert surface.get_data() == new.get_data()
+    finally:
+        os.chdir(old_dir)
+        shutil.rmtree(dirname)
+
+
 def test_scaled_font_get_ctm():
     surface = cairo.ImageSurface(0, 10, 10)
     ctx = cairo.Context(surface)
@@ -44,12 +78,6 @@ def test_scaled_font_get_font_options():
     assert isinstance(font_options, cairo.FontOptions)
 
 
-def test_ps_surface_get_levels():
-    levels = cairo.PSSurface.get_levels()
-    assert isinstance(levels, list)
-    assert all(isinstance(v, int) for v in levels)
-
-
 def test_ps_surface_level_to_string():
     level_id = cairo.PSSurface.level_to_string(cairo.PS_LEVEL_2)
     assert isinstance(level_id, str)
@@ -60,7 +88,7 @@ def test_surface_has_show_text_glyphs():
     surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 100, 100)
     assert not surface.has_show_text_glyphs()
     surface.finish()
-    with pytest.raises(cairo.Error) as excinfo:
+    with pytest.raises(cairo.Error):
         surface.has_show_text_glyphs()
 
 
@@ -95,12 +123,6 @@ def test_surface_create_similar_image():
     assert image.get_height() == 42
 
 
-def test_pdf_get_versions():
-    versions = cairo.PDFSurface.get_versions()
-    assert isinstance(versions, list)
-    assert all(isinstance(v, int) for v in versions)
-
-
 def test_pdf_surface_restrict_to_version():
     surface = cairo.PDFSurface(None, 10, 10)
     surface.restrict_to_version(cairo.PDF_VERSION_1_4)
@@ -123,6 +145,8 @@ def test_error_context():
         ctx.restore()
     error = excinfo.value
     assert error.status == cairo.STATUS_INVALID_RESTORE
+    assert error.status == cairo.Status.INVALID_RESTORE
+    assert isinstance(error.status, cairo.Status)
     assert str(error)
 
 
@@ -549,12 +573,6 @@ def test_surface_mime_data_for_pdf():
     assert jpeg_bytes in file_like.getvalue()
 
 
-def test_svg_surface_get_versions():
-    versions = cairo.SVGSurface.get_versions()
-    assert isinstance(versions, list)
-    assert all(isinstance(v, int) for v in versions)
-
-
 def test_svg_version_to_string():
     ver = cairo.SVGSurface.version_to_string(cairo.SVG_VERSION_1_1)
     assert ver and isinstance(ver, str)
